==> timemory-3.0.0rc2.tar.gz: timemory-3.0.0rc2/cmake/Scripts/gperf-cpu-profile.sh <==
#!/bin/bash

EXE=$(basename ${1})
DIR=cpu.prof.${EXE}
mkdir -p ${DIR}

# gperf settings
: ${N:=0}
: ${GPERF_PROFILE:=""}
: ${GPERF_PROFILE_BASE:=${DIR}/gperf}
: ${MALLOCSTATS:=1}
: ${CPUPROFILE_FREQUENCY:=250}
: ${CPUPROFILE_REALTIME:=1}

# rendering settings
: ${INTERACTIVE:=0}
: ${IMG_FORMAT:="jpeg"}
#: ${DOT_ARGS:='-Gsize=24,24\! -Gdpi=200'}
: ${DOT_ARGS:=""}
: ${PPROF_ARGS:="--no_strip_temp --functions"}

if [ "$(uname)" = "Darwin" ]; then
    if [ "${IMG_FORMAT}" = "jpeg" ]; then
        IMG_FORMAT="jpg"
    fi
fi
run-verbose()
{
    echo "${@}" 1>&2
    eval ${@}
}

while [ -z "${GPERF_PROFILE}" ]
do
    TEST_FILE=${GPERF_PROFILE_BASE}.${N}
    if [ ! -f "${TEST_FILE}" ]; then
        GPERF_PROFILE=${TEST_FILE}
    fi
    N=$((${N}+1))
done

export MALLOCSTATS
export CPUPROFILE_FREQUENCY
export CPUPROFILE_REALTIME

echo -e "\n\t--> Outputting profile to '${GPERF_PROFILE}'...\n"

# remove profile file if unsucessful execution
cleanup-failure() { set +v ; echo "failure"; rm -f ${GPERF_PROFILE}; }
trap cleanup-failure SIGHUP SIGINT SIGQUIT SIGILL SIGABRT SIGKILL

ADD_LIBS()
{
    for i in $@
    do
        if [ -z "${ADD_LIB_LIST}" ]; then
            ADD_LIB_LIST="--add_lib=${i}"
        else
            ADD_LIB_LIST="${ADD_LIB_LIST} --add_lib=${i}"
        fi
    done
}

ADD_PRELOAD()
{
    for i in $@
    do
        if [ -z "${LIBS}" ]; then
            LIBS=${i}
        else
            LIBS="${LIBS}:${i}"
        fi
    done
}

# configure pre-loading of profiler library
PROJECT_LIBRARIES="$(find $PWD | egrep 'libtimemory|libctimemory' | egrep -v '\.a$|\.dSYM')"
ADD_LIBS ${PROJECT_LIBRARIES}
ADD_PRELOAD ${PROJECT_LIBRARIES}
if [ "$(uname)" = "Darwin" ]; then
    ADD_PRELOAD $(otool -L ${1} | egrep 'profiler' | awk '{print $1}')
    LIBS=$(echo ${LIBS} | sed 's/^://g')
    if [ -n "${LIBS}" ]; then
        export DYLD_FORCE_FLAT_NAMESPACE=1
        export DYLD_INSERT_LIBRARIES=${LIBS}
        echo "DYLD_INSERT_LIBRARIES=${DYLD_INSERT_LIBRARIES}"
    fi
else
    ADD_PRELOAD $(ldd ${1} | egrep 'profiler' | awk '{print $(NF-1)}')
    LIBS=$(echo ${LIBS} | sed 's/^://g')
    if [ -n "${LIB}" ]; then
        export LD_PRELOAD=${LIBS}
        echo "LD_PRELOAD=${LD_PRELOAD}"
    fi
fi

set -e
# run the application
eval CPUPROFILE_FREQUENCY=${CPUPROFILE_FREQUENCY} CPUPROFILE=${GPERF_PROFILE} $@ | tee ${GPERF_PROFILE}.log
set +e

echo-dart-measurement()
{
    local _NAME=${1}
    local _TYPE=${2}
    local _PATH=${3}
    echo "<DartMeasurementFile name=\"${_NAME}\" type=\"image/${_TYPE}\">${_PATH}</DartMeasurementFile>"
}

# generate the results
EXT=so
if [ "$(uname)" = "Darwin" ]; then EXT=dylib; fi
if [ -f "${GPERF_PROFILE}" ]; then
    : ${PPROF:=$(which google-pprof)}
    : ${PPROF:=$(which pprof)}
    if [ -n "${PPROF}" ]; then
        run-verbose ${PPROF} --text ${ADD_LIB_LIST} ${PPROF_ARGS} ${1} ${GPERF_PROFILE} 1> ${GPERF_PROFILE}.txt
        run-verbose ${PPROF} --text --cum ${ADD_LIB_LIST} ${PPROF_ARGS} ${1} ${GPERF_PROFILE} 1> ${GPERF_PROFILE}.cum.txt
        # if dot is available
        if [ -n "$(which dot)" ]; then
            run-verbose ${PPROF} --dot ${ADD_LIB_LIST} ${PPROF_ARGS} ${1} ${GPERF_PROFILE} 1> ${GPERF_PROFILE}.dot
            run-verbose dot ${DOT_ARGS} -T${IMG_FORMAT} ${GPERF_PROFILE}.dot -o ${GPERF_PROFILE}.${IMG_FORMAT}
            echo-dart-measurement ${GPERF_PROFILE}.${IMG_FORMAT} ${IMG_FORMAT} ${PWD}/${GPERF_PROFILE}.${IMG_FORMAT}
            if [ -f ./gprof2dot.py ]; then
                run-verbose ${PPROF} --callgrind ${ADD_LIB_LIST} ${PPROF_ARGS} ${1} ${GPERF_PROFILE} 1> ${GPERF_PROFILE}.callgrind
                run-verbose python ./gprof2dot.py --format=callgrind --output=${GPERF_PROFILE}.callgrind.dot ${GPERF_PROFILE}.callgrind
                run-verbose dot ${DOT_ARGS} -T${IMG_FORMAT} ${GPERF_PROFILE}.callgrind.dot -o ${GPERF_PROFILE}.callgrind.${IMG_FORMAT}
                echo-dart-measurement ${GPERF_PROFILE}.callgrind ${IMG_FORMAT} ${PWD}/${GPERF_PROFILE}.callgrind.${IMG_FORMAT}
            fi
        fi
        if [ "${INTERACTIVE}" -gt 0 ]; then
            run-verbose ${PPROF} ${ADD_LIB_LIST} ${PPROF_ARGS} ${1} ${GPERF_PROFILE}
        fi
    else
        echo -e "google-pprof/pprof not found!"
    fi
else
    echo -e "profile file \"${GPERF_PROFILE}\" not found!"
fi

==> timemory-3.1.0.dev0.tar.gz: timemory-3.1.0.dev0/cmake/Scripts/gperf-cpu-profile.sh <==
#!/bin/bash

EXE=$(basename ${1})
DIR=cpu.prof.${EXE}
mkdir -p ${DIR}

# gperf settings
: ${N:=0}
: ${GPERF_PROFILE:=""}
: ${GPERF_PROFILE_BASE:=${DIR}/gperf}
: ${MALLOCSTATS:=1}
: ${CPUPROFILE_FREQUENCY:=250}
: ${CPUPROFILE_REALTIME:=1}

# rendering settings
: ${INTERACTIVE:=0}
: ${IMG_FORMAT:="jpeg"}
#: ${DOT_ARGS:='-Gsize=24,24\! -Gdpi=200'}
: ${DOT_ARGS:=""}
: ${PPROF_ARGS:="--no_strip_temp --functions"}

if [ "$(uname)" = "Darwin" ]; then
    if [ "${IMG_FORMAT}" = "jpeg" ]; then
        IMG_FORMAT="jpg"
    fi
fi
run-verbose()
{
    echo "${@}" 1>&2
    eval ${@}
}

while [ -z "${GPERF_PROFILE}" ]
do
    TEST_FILE=${GPERF_PROFILE_BASE}.${N}
    if [ ! -f "${TEST_FILE}" ]; then
        GPERF_PROFILE=${TEST_FILE}
    fi
    N=$((${N}+1))
done

export MALLOCSTATS
export CPUPROFILE_FREQUENCY
export CPUPROFILE_REALTIME

echo -e "\n\t--> Outputting profile to '${GPERF_PROFILE}'...\n"

# remove profile file if unsucessful execution
cleanup-failure() { set +v ; echo "failure"; rm -f ${GPERF_PROFILE}; }
trap cleanup-failure SIGHUP SIGINT SIGQUIT SIGILL SIGABRT SIGKILL

ADD_LIBS()
{
    for i in $@
    do
        if [ -z "${ADD_LIB_LIST}" ]; then
            ADD_LIB_LIST="--add_lib=${i}"
        else
            ADD_LIB_LIST="${ADD_LIB_LIST} --add_lib=${i}"
        fi
    done
}

ADD_PRELOAD()
{
    for i in $@
    do
        if [ -z "${LIBS}" ]; then
            LIBS=${i}
        else
            LIBS="${LIBS}:${i}"
        fi
    done
}

# configure pre-loading of profiler library
PROJECT_LIBRARIES="$(find $PWD | egrep 'libtimemory|libctimemory' | egrep -v '\.a$|\.dSYM')"
ADD_LIBS ${PROJECT_LIBRARIES}
ADD_PRELOAD ${PROJECT_LIBRARIES}
if [ "$(uname)" = "Darwin" ]; then
    ADD_PRELOAD $(otool -L ${1} | egrep 'profiler' | awk '{print $1}')
    LIBS=$(echo ${LIBS} | sed 's/^://g')
    if [ -n "${LIBS}" ]; then
        export DYLD_FORCE_FLAT_NAMESPACE=1
        export DYLD_INSERT_LIBRARIES=${LIBS}
        echo "DYLD_INSERT_LIBRARIES=${DYLD_INSERT_LIBRARIES}"
    fi
else
    ADD_PRELOAD $(ldd ${1} | egrep 'profiler' | awk '{print $(NF-1)}')
    LIBS=$(echo ${LIBS} | sed 's/^://g')
    if [ -n "${LIB}" ]; then
        export LD_PRELOAD=${LIBS}
        echo "LD_PRELOAD=${LD_PRELOAD}"
    fi
fi

set -e
# run the application
eval CPUPROFILE_FREQUENCY=${CPUPROFILE_FREQUENCY} CPUPROFILE=${GPERF_PROFILE} $@ | tee ${GPERF_PROFILE}.log
set +e

echo-dart-measurement()
{
    local _NAME=${1}
    local _TYPE=${2}
    local _PATH=${3}
    echo "<DartMeasurementFile name=\"${_NAME}\" type=\"image/${_TYPE}\">${_PATH}</DartMeasurementFile>"
}

# generate the results
EXT=so
if [ "$(uname)" = "Darwin" ]; then EXT=dylib; fi
if [ -f "${GPERF_PROFILE}" ]; then
    : ${PPROF:=$(which google-pprof)}
    : ${PPROF:=$(which pprof)}
    if [ -n "${PPROF}" ]; then
        run-verbose ${PPROF} --text ${ADD_LIB_LIST} ${PPROF_ARGS} ${1} ${GPERF_PROFILE} 1> ${GPERF_PROFILE}.txt
        run-verbose ${PPROF} --text --cum ${ADD_LIB_LIST} ${PPROF_ARGS} ${1} ${GPERF_PROFILE} 1> ${GPERF_PROFILE}.cum.txt
        # if dot is available
        if [ -n "$(which dot)" ]; then
            run-verbose ${PPROF} --dot ${ADD_LIB_LIST} ${PPROF_ARGS} ${1} ${GPERF_PROFILE} 1> ${GPERF_PROFILE}.dot
            run-verbose dot ${DOT_ARGS} -T${IMG_FORMAT} ${GPERF_PROFILE}.dot -o ${GPERF_PROFILE}.${IMG_FORMAT}
            echo-dart-measurement ${GPERF_PROFILE}.${IMG_FORMAT} ${IMG_FORMAT} ${PWD}/${GPERF_PROFILE}.${IMG_FORMAT}
            # if [ -f ./gprof2dot.py ]; then
            #    run-verbose ${PPROF} --callgrind ${ADD_LIB_LIST} ${PPROF_ARGS} ${1} ${GPERF_PROFILE} 1> ${GPERF_PROFILE}.callgrind
            #    run-verbose python ./gprof2dot.py --format=callgrind --output=${GPERF_PROFILE}.callgrind.dot ${GPERF_PROFILE}.callgrind
            #    run-verbose dot ${DOT_ARGS} -T${IMG_FORMAT} ${GPERF_PROFILE}.callgrind.dot -o ${GPERF_PROFILE}.callgrind.${IMG_FORMAT}
            #    echo-dart-measurement ${GPERF_PROFILE}.callgrind ${IMG_FORMAT} ${PWD}/${GPERF_PROFILE}.callgrind.${IMG_FORMAT}
            #fi
        fi
        if [ "${INTERACTIVE}" -gt 0 ]; then
            run-verbose ${PPROF} ${ADD_LIB_LIST} ${PPROF_ARGS} ${1} ${GPERF_PROFILE}
        fi
    else
        echo -e "google-pprof/pprof not found!"
    fi
else
    echo -e "profile file \"${GPERF_PROFILE}\" not found!"
fi
